<?php

define('PLUGIN_UNIQUE_ID', 'MTI2Mw==');

if ( ! class_exists( 'FSSC_License_Manager_Client' ) ) {
 
    class FSSC_License_Manager_Client {
     	private $api_endpoint;
     	private $product_id;
     	private $type;
     	private $text_domain;
   
	    public function __construct( $product_id, $text_domain, $api_url, $type = 'plugins') {
	    	$this->product_id = $product_id;
	    	$this->api_endpoint = base64_decode($api_url);
	    	$this->text_domain = $text_domain;
	    	$this->type = $type;
	    	$this->slug = FSSC_PLUGIN_SLUG;
	    	
	    	// Add a nag text for reminding the user to save the license information
			add_action( 'admin_notices', array( $this, 'show_admin_notices' ) );
			
			if ( $type == 'plugins' ) {
			    // Check for updates (for themes)
			    add_filter( 'pre_set_site_transient_update_plugins', array( $this, 'check_for_update' ) );
			}
	    }

		/**
		 * @return string The name of the settings field storing all license manager settings.
		 */
		protected function get_settings_field_name() {
		    return '_fssc_licence_settings';
		}
		 
	    /**
		 * If the license has not been configured properly, display an admin notice.
		 */
		public function show_admin_notices() {
			
		    $options = get_option( $this->get_settings_field_name() );
		 	
		    if ( !$options || ! isset( $options['email'] ) || ! isset( $options['license_key'] ) ||
		        $options['email'] == '' || $options['license_key'] == '' ) {
		 
		        $msg = __( 'Please enter your email and license key to enable updates to '.$this->product_id, $this->text_domain );
		        ?>
		            <div class="update-nag">
		                <p><?php echo $msg; ?></p>
		                <p>
		                    <a href="<?php echo admin_url( 'admin.php?page=fs-licence-settings' ); ?>">
		                        <?php _e( 'Complete the setup now.', $this->text_domain ); ?>
		                    </a>
		                </p>
		            </div>
		        <?php
		    }
		}

		//
		// API HELPER FUNCTIONS
		//
		 
		/**
		 * Makes a call to the WP License Manager API.
		 *
		 * @param $method   String  The API action to invoke on the license manager site
		 * @param $params   array   The parameters for the API call
		 * @return          array   The API response
		 */
		private function call_api( $action, $params ) {
		    $url = $this->api_endpoint . '/' . $action;
		    
		    // Append parameters for GET request
		    $url .= '?' . http_build_query( $params );

		    // Send the request
		    $response = wp_remote_get( $url );
		    if ( is_wp_error( $response ) ) {
		        return false;
		    }
		         
		    $response_body = wp_remote_retrieve_body( $response );
		    $result = json_decode( $response_body );

		    //echo "<pre>"; print_r($result); echo "</pre>";
		     
		    return $result;
		}

		/**
		 * Checks the API response to see if there was an error.
		 *
		 * @param $response mixed|object    The API response to verify
		 * @return bool     True if there was an error. Otherwise false.
		 */
		private function is_api_error( $response ) {
		    if ( $response === false ) {
		        return true;
		    }
		 
		    if ( ! is_object( $response ) ) {
		        return true;
		    }
		 
		    if ( isset( $response->error ) ) {
		        return true;
		    }
		 
		    return false;
		}

		/**
		 * Calls the License Manager API to get the license information for the
		 * current product.
		 *
		 * @return object|bool   The product data, or false if API call fails.
		 */
		public function get_license_info() {
		    $options = get_option( $this->get_settings_field_name() );
		    if ( ! isset( $options['email'] ) || ! isset( $options['license_key'] ) ) {
		        // User hasn't saved the license to settings yet. No use making the call.
		        return false;
		    }
		    $id = PLUGIN_UNIQUE_ID;
		 
		    $info = $this->call_api(
		        'info',
		        array(
		            'p' => $this->product_id,
		            'e' => $options['email'],
		            'l' => $options['license_key'],
		            'id' => $id
		        )
		    );
		    return $info;
		}


		/**
		 * Checks the license manager to see if there is an update available for this theme.
		 *
		 * @return object|bool  If there is an update, returns the license information.
		 *                      Otherwise returns false.
		 */
		public function is_update_available() {
		    $license_info = $this->get_license_info();
		    if ( $this->is_api_error( $license_info ) ) {
		        return false;
		    }
		 
		    if ( version_compare( $license_info->version, $this->get_local_version(), '>' ) ) {
		        return $license_info;
		    }
		    return false;
		}

		/**
		 * @return string   The theme / plugin version of the local installation.
		 */
		private function get_local_version() {
		    if ( $this->is_plugin() ) {
		        $plugin_data = get_plugin_data( FSSC_BASE_DIR. 'fs-schema.php');
		        return $plugin_data['Version'];
		    }
		}

		private function is_plugin() {
		    return $this->type == 'plugins';
		}

		/**
		 * The filter that checks if there are updates to the theme or plugin
		 * using the WP License Manager API.
		 *
		 * @param $transient    mixed   The transient used for WordPress
		 *                              theme / plugin updates.
		 *
		 * @return mixed        The transient with our (possible) additions.
		 */
		public function check_for_update( $transient ) {
		    if ( empty( $transient->checked ) ) {
		        return $transient;
		    }
		    $info = $this->is_update_available();
		    if ( $info !== false ) {
		        if ( $this->is_plugin() ) {
		            // Plugin updates will be added here.
		            $plugin_data = get_plugin_data( FSSC_BASE_DIR. 'fs-schema.php');
		            $plugin_slug = FSSC_PLUGIN_SLUG;
		            $plugin_path = FSSC_PLUGIN_PATH;

		            $transient->response[$plugin_path] = new stdClass();
		            $transient->response[$plugin_path]->new_version = $info->version;
		            $transient->response[$plugin_path]->package = $info->package_url;
		            $transient->response[$plugin_path]->url = $info->description_url;
		            $transient->response[$plugin_path]->plugin = $plugin_path;
		            $transient->response[$plugin_path]->slug = $plugin_slug; 
		        }  
		    }
		    return $transient;
		}
    }
}

$license_manager = new FSSC_License_Manager_Client(
    'FS Schema',
    'fs-schema-wordpress',
    'aHR0cHM6Ly9mdWR1Z28uY29tL3dwLWpzb24vcGx1Z2lucy9jaGVja3VwZGF0ZQ=='
);